'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _curry2 = require('lodash/curry');

var _curry3 = _interopRequireDefault(_curry2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (0, _curry3.default)(function (handler, socket) {
    if (typeof socket.on !== 'function') throw new Error("Please provide an instance of a socket.io connection");

    var resolve = function resolve(action) {
        return function (responseAction) {
            var _responseAction$meta = responseAction.meta,
                meta = _responseAction$meta === undefined ? {} : _responseAction$meta;

            socket.emit(_constants.SERVER_RESPONSE, {
                type: responseAction.type,
                payload: responseAction.payload,
                meta: _extends({}, meta, {
                    requestId: action.meta.requestId,
                    requestName: action.meta.requestName,
                    status: meta.status || _constants.COMPLETE_REQUEST,
                    error: meta.error
                })
            });
        };
    };

    var reject = function reject(action) {
        return function (e) {
            return socket.emit(_constants.SERVER_RESPONSE, {
                meta: {
                    status: _constants.FAIL_REQUEST,
                    requestId: action.meta.requestId,
                    requestName: action.meta.requestName,
                    error: e
                }
            });
        };
    };

    socket.on(_constants.SERVER_ACTION, function (action) {
        return handler({
            socket: socket,
            action: action,
            resolve: resolve(action),
            reject: reject(action)
        });
    });
});